#include <SPI.h>
#include <RH_RF95.h>
#include <RHDatagram.h>
#include <CayenneLPP.h>
#include <EEPROM.h>
#include <BME280I2C.h>
#include <Wire.h>
#include <Lora_Config.h>

///////////////////////////////////////////////////////////////////
// START CONFIGURATION
///////////////////////////////////////////////////////////////////
#define RF95_CLIENT_DEBUG
#define SERIAL_BAUD 115200
///////////////////////////////////////////////////////////////////
// END CONFIGURATION
///////////////////////////////////////////////////////////////////

#define EEADDR 128
#define DONE_PIN A1

Lora_Config data;

BME280I2C bme;    // Default : forced mode, standby time = 1000 ms, Oversampling = pressure ×1, temperature ×1, humidity ×1, filter off,
CayenneLPP lpp(51);
RH_RF95 rf95;
RHDatagram manager(rf95);

volatile float h; // Relative Humidity
volatile float t; // Temperature
volatile float p; // Presure
volatile float v; // Voltage
const int pwr_down_loops = 150;

float readVcc()
{
  long result;
  // Read 1.1V reference against AVcc
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Convert
  while (bit_is_set(ADCSRA, ADSC));
  result = ADCL;
  result |= ADCH << 8;
  result = 1126400L / result; // Back-calculate AVcc in mV
  return float(result) / 1000;
}

void setup() 
{
  Wire.begin();
  pinMode(DONE_PIN, OUTPUT);
  pinMode(DONE_PIN, LOW);

#ifdef RF95_CLIENT_DEBUG
  Serial.begin(SERIAL_BAUD);
#endif

  EEPROM.get(EEADDR, data);
#ifdef RF95_CLIENT_DEBUG
  Serial.println("=== Read EEPROM ===");
  Serial.print("node_addr: ");
  Serial.println(data.node_addr, DEC);
  Serial.print("gateway_addr: ");
  Serial.println(data.gateway_addr, DEC);
  Serial.print("lora_frequency: ");
  Serial.println(data.frequency, DEC);

  Serial.print("Initializing LORA module ");
#endif
  manager.setThisAddress(data.node_addr);
  if (manager.init())
  {
#ifdef RF95_CLIENT_DEBUG
    Serial.println("OK");
#endif
    rf95.setFrequency(data.frequency);
  }
  else
  {
#ifdef RF95_CLIENT_DEBUG
    Serial.print("Failed");
#endif
    while(1);
  }
#ifdef RF95_CLIENT_DEBUG
  Serial.print("Initializing BME280 sensor ");
#endif
  if (bme.begin())
  {
#ifdef RF95_CLIENT_DEBUG
    Serial.println("OK");
#endif
  }
  else
  {
#ifdef RF95_CLIENT_DEBUG
    Serial.println("failed");
#endif
  }
  delay(1000);
}

void loop()
{
  v = readVcc();
  
#ifdef RF95_CLIENT_DEBUG
  Serial.print("Reading sensor data:\n");
#endif
   float t(NAN), h(NAN), p(NAN);

   BME280::TempUnit tempUnit(BME280::TempUnit_Celsius);
   BME280::PresUnit presUnit(BME280::PresUnit_Pa);
   bme.read(p, t, h, tempUnit, presUnit);
#ifdef RF95_CLIENT_DEBUG
  Serial.print("\n\tTemperature: ");
  Serial.print(t);
  Serial.println("C\n");
  Serial.print("\tRelative Humidity: ");
  Serial.print(h);
  Serial.println("%\n");
  Serial.print("\tBarometric Pressure: ");
  Serial.print(p);
  Serial.println("\n");

  Serial.println("Encoding results");
#endif
  lpp.reset();
  lpp.addTemperature(1, t);
  lpp.addRelativeHumidity(2, h);
  lpp.addVoltage(3, v);
  lpp.addBarometricPressure(4, p);

#ifdef RF95_CLIENT_DEBUG
  Serial.print("Sende Sensordaten ");
#endif
  if (manager.sendto(lpp.getBuffer(), lpp.getSize(), data.gateway_addr))
  {
#ifdef RF95_CLIENT_DEBUG
    Serial.print("OK\n");
#endif
    manager.waitPacketSent();
  }
#ifdef RF95_CLIENT_DEBUG
  Serial.println("Going to sleep\n");
#endif
//  while (1) {
    digitalWrite(DONE_PIN, LOW);
//    delay(1);
    digitalWrite(DONE_PIN, HIGH);  
//    delay(1);
//  }
}
